# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2017, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from functools import wraps

import prestans.exception


class Base(object):
    """Base class for an auth provider, this should be overridden to suit
    each application."""

    def __init__(self):
        self._debug = False
        self._request = None

    @property
    def debug(self):
        """Getter for debug property"""
        return self._debug

    @debug.setter
    def debug(self, value):
        """Setter for debug property"""
        self._debug = value

    @property
    def request(self):
        """Getter for request property"""
        return self._request

    @request.setter
    def request(self, value):
        """Setter for request property"""
        self._request = value

    def current_user_has_role(self, role_name):
        """Override this method to check if the given role is allowed access"""
        raise NotImplementedError

    def is_authenticated_user(self):
        """Override this method to check if a user is logged in"""
        raise NotImplementedError

    def is_authorized_user(self, config):
        """Override this method to check if a user has access"""
        raise NotImplementedError

    def get_current_user(self):
        """Override this method to provide reference to logged in user"""
        raise NotImplementedError


def login_required(http_method_handler):
    """
    provides a decorator for RESTRequestHandler methods to check for authenticated users

    RESTRequestHandler subclass must have a auth_context instance, refer to prestans.auth
    for the parent class definition.

    If decorator is used and no auth_context is provided the client will be denied access.

    Handler will return a 401 Unauthorized if the user is not logged in, the service does
    not redirect to login handler page, this is the client's responsibility.

    auth_context_handler instance provides a message called get_current_user, use this
    to obtain a reference to an authenticated user profile.

    If all goes well, the original handler definition is executed.
    """

    @wraps(http_method_handler)
    def secure_http_method_handler(self, *args):

        if not self.__provider_config__.authentication:
            _message = ("Service available to authenticated users only, "
                        "no auth context provider set in handler")
            authentication_error = prestans.exception.AuthenticationError(_message)
            authentication_error.request = self.request
            raise authentication_error

        if not self.__provider_config__.authentication.is_authenticated_user():
            authentication_error = prestans.exception.AuthenticationError()
            authentication_error.request = self.request
            raise authentication_error

        http_method_handler(self, *args)

    return secure_http_method_handler


def role_required(role_name=None):
    """
    Authenticates a HTTP method handler based on a provided role

    With a little help from Peter Cole's Blog
    http://mrcoles.com/blog/3-decorator-examples-and-awesome-python/

    """

    def _role_required(http_method_handler):

        @wraps(http_method_handler)
        def secure_http_method_handler(self, *args):

            # Role name must be provided
            if role_name is None:
                _message = "Role name must be provided"
                authorization_error = prestans.exception.AuthorizationError(_message)
                authorization_error.request = self.request
                raise authorization_error

            # Authentication context must be set
            if not self.__provider_config__.authentication:
                _message = ("Service available to authenticated users only, "
                            "no auth context provider set in handler")
                authentication_error = prestans.exception.AuthenticationError(_message)
                authentication_error.request = self.request
                raise authentication_error

            # Check for the role by calling current_user_has_role
            if not self.__provider_config__.authentication.current_user_has_role(role_name):
                authorization_error = prestans.exception.AuthorizationError(role_name)
                authorization_error.request = self.request
                raise authorization_error

            http_method_handler(self, *args)

        return wraps(http_method_handler)(secure_http_method_handler)

    return _role_required


def access_required(config=None):
    """
    Authenticates a HTTP method handler based on a custom set of arguments
    """

    def _access_required(http_method_handler):

        def secure_http_method_handler(self, *args):

            # Authentication context must be set
            if not self.__provider_config__.authentication:
                _message = ("Service available to authenticated users only, "
                            "no auth context provider set in handler")
                authentication_error = prestans.exception.AuthenticationError(_message)
                authentication_error.request = self.request
                raise authentication_error

            # Check for access by calling is_authorized_user
            if not self.__provider_config__.authentication.is_authorized_user(config):
                _message = "Service available to authorized users only"
                authorization_error = prestans.exception.AuthorizationError(_message)
                authorization_error.request = self.request
                raise authorization_error

            http_method_handler(self, *args)

        return wraps(http_method_handler)(secure_http_method_handler)

    return _access_required



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2017, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import logging
import unittest

import prestans.rest

LOGGER_MCLOGFACE = logging.Logger("temp", level='ERROR')
LOGGER_MCLOGFACE.disabled = 50  # silence the logger


class MyModel(prestans.types.Model):
    id = prestans.types.Integer()


class _UserHandler(prestans.rest.RequestHandler):
    __parser_config__ = prestans.parser.Config(
        GET=prestans.parser.VerbConfig(
            response_template=MyModel(),
            response_attribute_filter_default_value=True
        )
    )

    def get(self, id):
        model = MyModel()
        model.id = id
        self.response.body = model


class MockStartResponse:
    @classmethod
    def __call__(cls, status, response_headers, exc_info=None):
        pass


class RequestRouterTest(unittest.TestCase):
    def test_script_alias_match_with_global_match_group_should_not_pass_call(self):
        """
        WSGIScriptAliasMatch /mountpoint(.*) script.wsgi

        with router r"/mountpoint/some/path/([0-9]+"
        """

        expected_value = 123

        self._test_routing_behavour(environ={
            "REQUEST_METHOD": prestans.http.VERB.GET,
            "SCRIPT_NAME": "/mountpoint/some/path/{}".format(expected_value),
            "PATH_INFO": "",
            "wsgi.version": (1, 0),
            "wsgi.url_scheme": "http",
            "wsgi.input": None,  # shouldn't be called
            "SERVER_NAME": "localhost",
            "SERVER_PORT": "1234"
        }, should_pass=False)

    def test_script_alias_match_with_match_group_should_pass_call(self):
        """
        WSGIScriptAliasMatch /mountpoint(.*) script.wsgi$1

        with router ``TestRequestRouter``
        :return:
        """
        self._test_routing_behavour(environ={
            "REQUEST_METHOD": prestans.http.VERB.GET,
            "SCRIPT_NAME": "/mountpoint",
            "PATH_INFO": "/some/path/{}".format(123),
            "wsgi.url_scheme": "http",
            "SERVER_NAME": "localhost",
            "SERVER_PORT": "1234"
        })

    def test_script_alias_match_with_solidus_should_pass_call(self):
        """
        WSGIScriptAliasMatch /mountpoint/(.*) script.wsgi/$1
        :return:
        """
        self._test_routing_behavour(environ={
            "REQUEST_METHOD": prestans.http.VERB.GET,
            "SCRIPT_NAME": "/mountpoint/",
            "PATH_INFO": "/some/path/{}".format(123),
            "wsgi.url_scheme": "http",
            "SERVER_NAME": "localhost",
            "SERVER_PORT": "1234"
        })

    def test_router_should_not_crash_with_no_path_info_field(self):
        """
        Test the ability to accept blank PATH_INFO as per specification PEP 3333

        :return:
        """
        expected_value = 123

        self._test_routing_behavour(environ={
            "REQUEST_METHOD": prestans.http.VERB.GET,
            "SCRIPT_NAME": "/mountpoint/some/path/{}".format(123),
            "wsgi.url_scheme": "http",
            "SERVER_NAME": "localhost",
            "SERVER_PORT": "1234"
        }, match=r"/mountpoint/some/path/([0-9]+)", should_pass=False)

    def test_router_should_not_crash_with_no_script_name_field(self):
        """
        Test the ability to accept a blank SCRIPT_NAME as per spec PEP 3333

        `<https://www.python.org/dev/peps/pep-3333/#environ-variables>`
        :return:
        """

        expected_value = 123

        self._test_routing_behavour(environ={
            "REQUEST_METHOD": prestans.http.VERB.GET,
            "PATH_INFO": "/some/path/{}".format(123),
            "wsgi.url_scheme": "http",
            "SERVER_NAME": "localhost",
            "SERVER_PORT": "1234"})

    def _test_routing_behavour(self, environ, should_pass=True, expected_value=123, match=r"/some/path/([0-9]+)"):

        test_router = prestans.rest.RequestRouter([
            (match, _UserHandler)
        ], application_name="test-router", logger=LOGGER_MCLOGFACE)

        response = test_router(environ=environ, start_response=MockStartResponse.__call__)
        response_parsed = prestans.deserializer.JSON().loads(response[0])
        if should_pass:
            self.assert_should_find_route(environ, expected_value, match, response_parsed)
        else:
            self.assert_should_not_find_route(environ, match, response_parsed)

    def assert_should_not_find_route(self, environ, match, response_parsed):
        if 'id' in response_parsed.keys():
            self.fail(self.gen_route_match_failure_message(environ, match, assertion="SHOULD NOT MATCH"))

    def assert_should_find_route(self, environ, expected_value, match, response_parsed):
        if 'code' in response_parsed.keys() and response_parsed['code'] == 404:
            self.fail(self.gen_route_match_failure_message(environ, match, assertion="SHOULD MATCH"))
        else:
            self.assertEqual(response_parsed['id'], expected_value,
                             "expected id field with value of {}".format(expected_value))

    def gen_route_match_failure_message(self, environ, match, assertion="SHOULD"):
        return "SCRIPT_NAME[{script}] and PATH_INFO[{path}] {assertion} route[{route}]".format(
            script=environ.get("SCRIPT_NAME", ""),
            path=environ.get("PATH_INFO", ""),
            route=match,
            assertion=assertion
        )


if __name__ == '__main__':
    unittest.main()



from prestans.parser import VerbConfig
from prestans.types import Array
from prestans.types import Boolean
from prestans.types import Float
from prestans.types import Integer
from prestans.types import String


def test_verb_config_boolean_array():
    boolean_array = Array(element_template=Boolean())
    VerbConfig(response_template=boolean_array)


def test_verb_config_float_array():
    float_array = Array(element_template=Float())
    VerbConfig(response_template=float_array)


def test_verb_config_integer_array():
    integer_array = Array(element_template=Integer())
    VerbConfig(response_template=integer_array)


def test_verb_config_string_array():
    string_array = Array(element_template=String())
    VerbConfig(response_template=string_array)


# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2017, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import unittest

import prestans.types
import prestans.exception


class StringTypeUnitTest(unittest.TestCase):

    def setUp(self):
        self.required = prestans.types.String(required=True)
        self.not_required = prestans.types.String(required=False)
        self.types = prestans.types.String()
        self.default = prestans.types.String(default="orange")
        self.length = prestans.types.String(min_length=5, max_length=7)
        self.format = prestans.types.String(format="[0-9]{2}[a-z]{5}[0-9]{3}")
        self.choices = prestans.types.String(choices=["apple", "banana"])

    def test_required(self):
        self.assertRaises(prestans.exception.RequiredAttributeError, self.required.validate, None)
        self.assertRaises(prestans.exception.RequiredAttributeError, self.required.validate, "")
        self.assertEqual(self.required.validate("apple"), "apple")

        self.assertEqual(self.not_required.validate("apple"), "apple")
        self.assertEqual(self.not_required.validate(""), "")
        self.assertEqual(self.not_required.validate(None), None)

    def test_types(self):
        self.assertEqual(self.types.validate("orange"), "orange")
        self.assertEqual(self.types.validate(1), "1")
        self.assertEqual(self.types.validate(1.0), "1.0")
        self.assertEqual(self.types.validate(True), "True")

    def test_default(self):
        self.assertEqual(self.default.validate(None), "orange")
        self.assertEqual(self.default.validate("apple"), "apple")

    def test_min_length(self):
        self.assertRaises(prestans.exception.MinimumLengthError, self.length.validate, "1234")
        self.assertEqual(self.length.validate("12345"), "12345")

    def test_max_length(self):
        self.assertRaises(prestans.exception.MaximumLengthError, self.length.validate, "123456789")
        self.assertEqual(self.length.validate("1234567"), "1234567")

    def test_format(self):
        self.assertRaises(prestans.exception.InvalidFormatError, self.format.validate, "cat")
        self.assertRaises(prestans.exception.InvalidFormatError, self.format.validate, "ab45678as")
        self.assertEqual(self.format.validate("12abcde123"), "12abcde123")
        self.assertEqual(self.format.validate("89uwxyz789"), "89uwxyz789")

    def test_choices(self):
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, "orange")
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, "grape")

        self.assertEqual(self.choices.validate("apple"), "apple")
        self.assertEqual(self.choices.validate("banana"), "banana")

    def test_encoding(self):
        pass

    def tearDown(self):
        pass


class IntegerTypeUnitTest(unittest.TestCase):

    def setUp(self):
        self.types = prestans.types.Integer()
        self.required = prestans.types.Integer(required=True)
        self.not_required = prestans.types.Integer(required=False)
        self.default = prestans.types.Integer(default=6)
        self.positive_range = prestans.types.Integer(minimum=1, maximum=5)
        self.negative_range = prestans.types.Integer(minimum=-5, maximum=-1)
        self.zero_min_range = prestans.types.Integer(minimum=0)
        self.zero_max_range = prestans.types.Integer(maximum=0)
        self.choices = prestans.types.Integer(choices=[1,5])

    def test_types(self):
        self.assertRaises(prestans.exception.ParseFailedError, self.types.validate, "string")

    def test_required(self):
        self.assertRaises(prestans.exception.RequiredAttributeError, self.required.validate, None)
        self.assertEqual(self.required.validate(1), 1)

        self.assertEqual(self.not_required.validate(1), 1)
        self.assertEqual(self.not_required.validate(None), None)

    def test_default(self):
        self.assertEqual(self.default.validate(None), 6)
        self.assertEqual(self.default.validate(5), 5)

    def test_minimum(self):
        self.assertRaises(prestans.exception.LessThanMinimumError, self.positive_range.validate, -2)
        self.assertRaises(prestans.exception.LessThanMinimumError, self.positive_range.validate, -1)
        self.assertRaises(prestans.exception.LessThanMinimumError, self.positive_range.validate, 0)
        self.assertEqual(self.positive_range.validate(1), 1)
        self.assertEqual(self.positive_range.validate(2), 2)
        self.assertEqual(self.positive_range.validate(3), 3)

        self.assertRaises(prestans.exception.LessThanMinimumError, self.negative_range.validate, -8)
        self.assertRaises(prestans.exception.LessThanMinimumError, self.negative_range.validate, -7)
        self.assertRaises(prestans.exception.LessThanMinimumError, self.negative_range.validate, -6)
        self.assertEqual(self.negative_range.validate(-5), -5)
        self.assertEqual(self.negative_range.validate(-4), -4)
        self.assertEqual(self.negative_range.validate(-3), -3)

        self.assertRaises(prestans.exception.LessThanMinimumError, self.zero_min_range.validate, -1)
        self.assertEqual(self.zero_min_range.validate(0), 0)

    def text_maximum(self):
        self.assertEqual(self.positive_range.validate(0), 0)
        self.assertEqual(self.positive_range.validate(3), 3)
        self.assertEqual(self.positive_range.validate(4), 4)
        self.assertEqual(self.positive_range.validate(5), 5)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.positive_range.validate, 6)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.positive_range.validate, 7)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.positive_range.validate, 8)

        self.assertEqual(self.negative_range.validate(-3), -3)
        self.assertEqual(self.negative_range.validate(-2), -2)
        self.assertEqual(self.negative_range.validate(-1), -1)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.negative_range.validate, 0)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.negative_range.validate, 1)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.negative_range.validate, 2)

        self.assertEqual(self.zero_min_range.validate(-1), -1)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.zero_min_range.validate, 1)

    def test_choices(self):
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, 3)
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, 4)
        self.assertEqual(self.choices.validate(1), 1)
        self.assertEqual(self.choices.validate(5), 5)

    def tearDown(self):
        pass


class FloatTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self.types = prestans.types.Float()
        self.required = prestans.types.Float(required=True)
        self.not_required = prestans.types.Float(required=False)
        self.default = prestans.types.Float(default=6.0)
        self.positive_range = prestans.types.Float(minimum=1.0, maximum=5.0)
        self.negative_range = prestans.types.Float(minimum=-5.0, maximum=-1.0)
        self.choices = prestans.types.Float(choices=[1.0, 3.0])

    def test_types(self):
        self.assertRaises(prestans.exception.ParseFailedError, self.types.validate, "string")

    def test_required(self):
        self.assertRaises(prestans.exception.RequiredAttributeError, self.required.validate, None)
        self.assertEqual(self.required.validate(1.0), 1.0)

        self.assertEqual(self.not_required.validate(1.0), 1.0)
        self.assertEqual(self.not_required.validate(None), None)

    def test_default(self):
        self.assertEqual(self.default.validate(5.0), 5.0)
        self.assertEqual(self.default.validate(None), 6.0)

    def test_minimum(self):
        self.assertRaises(prestans.exception.LessThanMinimumError, self.positive_range.validate, 0.0)
        self.assertRaises(prestans.exception.LessThanMinimumError, self.positive_range.validate, 0.5)
        self.assertEqual(self.positive_range.validate(1.0), 1.0)
        self.assertEqual(self.positive_range.validate(1.5), 1.5)

        self.assertRaises(prestans.exception.LessThanMinimumError, self.negative_range.validate, -7.0)
        self.assertRaises(prestans.exception.LessThanMinimumError, self.negative_range.validate, -6.0)
        self.assertEqual(self.negative_range.validate(-5.0), -5.0)
        self.assertEqual(self.negative_range.validate(-4.0), -4.0)

    def text_maximum(self):
        self.assertEqual(self.positive_range.validate(4.0), 4.0)
        self.assertEqual(self.positive_range.validate(4.5), 4.5)
        self.assertEqual(self.positive_range.validate(5.0), 5.0)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.positive_range.validate, 6.5)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.positive_range.validate, 7.0)

        self.assertEqual(self.negative_range.validate(-3.0), -3.0)
        self.assertEqual(self.negative_range.validate(-2.0), -2.0)
        self.assertEqual(self.negative_range.validate(-1.0), -1.0)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.negative_range.validate, -0.5)
        self.assertRaises(prestans.exception.MoreThanMaximumError, self.negative_range.validate, 0.0)

    def test_choices(self):
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, 0.0)
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, 2.0)
        self.assertRaises(prestans.exception.InvalidChoiceError, self.choices.validate, 4.0)
        self.assertEqual(self.choices.validate(1.0), 1.0)
        self.assertEqual(self.choices.validate(3.0), 3.0)

    def tearDown(self):
        pass


class BooleanTypeUnitTest(unittest.TestCase):

    def setUp(self):
        self.types = prestans.types.Boolean()

        self.required = prestans.types.Boolean(required=True)
        self.not_required = prestans.types.Boolean(required=False)

        self.default_true = prestans.types.Boolean(default=True)
        self.default_false = prestans.types.Boolean(default=False)

    def test_types(self):
        self.types.validate("string")

    def test_required(self):
        self.assertRaises(prestans.exception.RequiredAttributeError, self.required.validate, None)

    def test_default(self):
        self.assertEqual(self.default_true.validate(None), True)
        self.assertEqual(self.default_true.validate(False), False)
        self.assertEqual(self.default_false.validate(None), False)
        self.assertEqual(self.default_false.validate(True), True)

    def tearDown(self):
        pass


class DataURLFileTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._data_url_file = prestans.types.DataURLFile(
            allowed_mime_types=[],
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        pass


class DateTimeTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._datetime = prestans.types.DateTime(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        pass


class DateTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._date = prestans.types.Date(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        pass


class TimeTypeUnitTest(unittest.TestCase):

    def setUp(self):
        
        self._time = prestans.types.Time(
            default=True,
            )

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        pass


class ArrayTypeUnitTest(unittest.TestCase):

    def setUp(self):        
        self._string_array = prestans.types.Array(
            element_template=prestans.types.String()
        )
        self._integer_array = prestans.types.Array(
            element_template=prestans.types.Integer()
        )
        self._boolean_array = prestans.types.Array(
            element_template=prestans.types.Boolean()
        )
        self._float_array = prestans.types.Array(
            element_template=prestans.types.Float()
        )

    def test_array_is_scalar_is_true_for_scalar_element_templates(self):
        self.assertTrue(self._integer_array.is_scalar)
        self.assertTrue(self._string_array.is_scalar)
        self.assertTrue(self._boolean_array.is_scalar)
        self.assertTrue(self._float_array.is_scalar)

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def test_append(self):
        pass

    def tearDown(self):
        pass

class ModelTypeUnitTest(unittest.TestCase):

    class TestModel(prestans.types.Model):

        string = prestans.types.String(required=True, max_length=10)

    def setUp(self):
        
        self._model = ModelTypeUnitTest.TestModel()

    def test_default(self):
        pass

    def test_choices(self):
        pass

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()


from prestans.parser import AttributeFilter
from prestans.types import Array
from prestans.types import Boolean
from prestans.types import Float
from prestans.types import Integer
from prestans.types import String


def test_from_model_boolean_array():
    boolean_array = Array(element_template=Boolean())
    AttributeFilter.from_model(model_instance=boolean_array, default_value=True)


def test_from_model_float_array():
    float_array = Array(element_template=Float())
    AttributeFilter.from_model(model_instance=float_array, default_value=True)


def test_from_model_integer_array():
    integer_array = Array(element_template=Integer())
    AttributeFilter.from_model(model_instance=integer_array, default_value=True)


def test_from_model_string_array():
    string_array = Array(element_template=String())
    AttributeFilter.from_model(model_instance=string_array, default_value=True)



#!/usr/bin/env python
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2017, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

from setuptools import setup, find_packages

from prestans import __version__

setup(name='prestans',
    version=__version__,
    description='A WSGI compliant REST micro-framework',
    url='https://github.com/anomaly/prestans.git',
    long_description="""
    Prestans is a REST micro-framework built right on top of WSGI, designed to perform and co-exists with other middleware and frameworks that you employ. 

    Our target audience are developers building large REST backends for use with pure Ajax (using a framework like Google Closure) or mobile applications. 

    Prestans is designed for you to "take as much or as little" as you like.

    Features:

    - Built right on top of WSGI, designed ground up to cooperate with other frameworks
    - Strong representative of REST philosophies leveraging HTTP headers and verbs
    - Support for multiple dialects, including formalised patterns for binary content 
    - Handlers maps HTTP verbs to implemented class methods, complimented  with custom 
    request parser and response writer (built on top of WebOb)
    - Unforgivingly strict parsing of requests and responses to ensure data integrity
    - Helper methods to ease transformation of persistent objects to REST responses
    - Pluggable integration to authentication

    """,
    download_url='https://github.com/anomaly/prestans/archive/' + __version__ + '.tar.gz',
    license='New BSD',
    author='Anomaly Software',
    author_email='support@anomaly.net.au',
    maintainer='Anomaly Software',
    maintainer_email='support@anomaly.net.au',
    platforms=['any'],
    scripts=['prestans/bin/pride'],
    packages=find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=['WebOb==1.4'],
    tests_require=['pytest'],
    setup_requires=['pytest-runner'],
    extras_require={
        "devel": ["Jinja2"]
    },
    include_package_data = True
)



# -*- coding: utf-8 -*-
#
#  prestans, A WSGI compliant REST micro-framework
#  http://prestans.org
#
#  Copyright (c) 2017, Anomaly Software Pty Ltd.
#  All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#      * Redistributions of source code must retain the above copyright
#        notice, this list of conditions and the following disclaimer.
#      * Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.
#      * Neither the name of Anomaly Software nor the
#        names of its contributors may be used to endorse or promote products
#        derived from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED. IN NO EVENT SHALL ANOMALY SOFTWARE BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

import logging
import unittest

import prestans.provider.auth
import prestans.rest


class BaseUnitTest(unittest.TestCase):

    def setUp(self):

        self.auth = prestans.provider.auth.Base()

    def test_debug(self):
        self.assertEqual(self.auth.debug, False)
        self.auth.debug = True
        self.assertEqual(self.auth.debug, True)

    def test_current_user_has_role(self):
        self.assertRaises(NotImplementedError, self.auth.current_user_has_role, "Admin")

    def test_is_authenticated_user(self):
        self.assertRaises(NotImplementedError, self.auth.is_authenticated_user)

    def test_is_authorized_user(self):
        self.assertRaises(NotImplementedError, self.auth.is_authorized_user, None)

    def test_get_current_user(self):
        self.assertRaises(NotImplementedError, self.auth.get_current_user)

    def tearDown(self):
        pass


class AdminRoleProvider(prestans.provider.auth.Base):
    
    def current_user_has_role(self, role_name):
        return role_name == "Admin"


class AuthenticatedProvider(prestans.provider.auth.Base):

    def is_authenticated_user(self):
        return True


class UnauthenticatedProvider(prestans.provider.auth.Base):

    def is_authenticated_user(self):
        return False


class AuthorizedProvider(prestans.provider.auth.Base):

    def is_authorized_user(self, config):
        return True


class UnauthorizedProvider(prestans.provider.auth.Base):

    def is_authorized_user(self, config):
        return False


class CustomUnitTest(unittest.TestCase):

    def setUp(self):

        self.admin_role = AdminRoleProvider()
        self.authenticated = AuthenticatedProvider()
        self.unauthenticated = UnauthenticatedProvider()
        self.authorized = AuthorizedProvider()
        self.unauthorized = UnauthorizedProvider()

    def test_debug(self):
        self.assertEqual(self.admin_role.debug, False)
        self.admin_role.debug = True
        self.assertEqual(self.admin_role.debug, True)

    def test_current_user_has_role(self):
        self.assertEqual(self.admin_role.current_user_has_role("Admin"), True)
        self.assertEqual(self.admin_role.current_user_has_role("Manager"), False)

    def test_is_authenticated_user(self):
        self.assertEqual(self.authenticated.is_authenticated_user(), True)
        self.assertEqual(self.unauthenticated.is_authenticated_user(), False)

    def test_is_authorized_user(self):
        self.assertEqual(self.authorized.is_authorized_user(None), True)
        self.assertEqual(self.unauthorized.is_authorized_user(None), False)

    def tearDown(self):
        pass


class AuthenticatedHandlerProvider(prestans.provider.auth.Base):
    
    def is_authenticated_user(self):
        return True

    def current_user_has_role(self, role_name):
        return role_name == "Admin"


class UnauthenticatedHandlerProvider(prestans.provider.auth.Base):
    
    def is_authenticated_user(self):
        return False


class HandlerWithoutProvider(prestans.rest.RequestHandler):

    @prestans.provider.auth.login_required
    def get(self):
        pass


class AuthenticatedHandler(prestans.rest.RequestHandler):
    
    __provider_config__ = prestans.provider.Config(
        authentication=AuthenticatedHandlerProvider()
    )

    @prestans.provider.auth.login_required
    def get(self):
        self.response.status = prestans.http.STATUS.NO_CONTENT

    @prestans.provider.auth.role_required("Admin")
    def post(self):
        self.response.status = prestans.http.STATUS.NO_CONTENT

    @prestans.provider.auth.role_required("Manager")
    def put(self):
        self.response.status = prestans.http.STATUS.NO_CONTENT


class UnauthenticatedHandler(prestans.rest.RequestHandler):
    
    __provider_config__ = prestans.provider.Config(
        authentication=UnauthenticatedHandlerProvider()
    )

    @prestans.provider.auth.login_required
    def get(self):
        self.response.status = prestans.http.STATUS.NO_CONTENT


def start_response(status, headers):
    pass


class HandlerUnitTest(unittest.TestCase):

    def setUp(self):

        logging.basicConfig()
        logger = logging.getLogger("prestans")

        charset="utf-8"
        serializers=[prestans.deserializer.JSON()]
        default_serializer=prestans.deserializer.JSON()

        self.get_environ = {"REQUEST_METHOD": prestans.http.VERB.GET}
        self.post_environ = {"REQUEST_METHOD": prestans.http.VERB.POST}
        self.put_environ = {"REQUEST_METHOD": prestans.http.VERB.PUT}

        get_request = prestans.rest.Request(
            environ=self.get_environ,
            charset=charset,
            logger=logger,
            deserializers=serializers,
            default_deserializer=default_serializer
        )

        post_request = prestans.rest.Request(
            environ=self.post_environ,
            charset=charset,
            logger=logger,
            deserializers=serializers,
            default_deserializer=default_serializer
        )

        put_request = prestans.rest.Request(
            environ=self.put_environ,
            charset=charset,
            logger=logger,
            deserializers=serializers,
            default_deserializer=default_serializer
        )

        response = prestans.rest.Response(
            charset=charset,
            logger=logger,
            serializers=serializers,
            default_serializer=default_serializer
        )

        self.authenticated_handler = AuthenticatedHandler(
            args=[],
            request=get_request,
            response=response,
            logger=logger,
            debug=True
        )

        self.unauthenticated_handler = UnauthenticatedHandler(
            args=[],
            request=get_request,
            response=response,
            logger=logger,
            debug=True
        )

        self.correct_role_handler = AuthenticatedHandler(
            args=[],
            request=post_request,
            response=response,
            logger=logger,
            debug=True
        )

        self.incorrect_role_handler = AuthenticatedHandler(
            args=[],
            request=put_request,
            response=response,
            logger=logger,
            debug=True
        )

        self.handler_without_provider = HandlerWithoutProvider(
            args=[],
            request=get_request,
            response=response,
            logger=logger,
            debug=True
        )

    def test_login_required(self):
        self.assertRaises(prestans.exception.AuthenticationError, self.handler_without_provider, self.get_environ, start_response)
        self.assertRaises(prestans.exception.AuthenticationError, self.unauthenticated_handler, self.get_environ, start_response)
        self.assertIsInstance(self.authenticated_handler(self.get_environ, start_response), list)

    def test_current_user_has_role(self):
        self.assertRaises(prestans.exception.AuthorizationError, self.incorrect_role_handler, self.post_environ, start_response)
        self.assertIsInstance(self.correct_role_handler(self.put_environ, start_response), list)

    def tearDown(self):
        pass